@@ -1,19 +1,18 @@
-use crate::codegen::accounts::{constraints, generics};
+use crate::codegen::accounts::{constraints, generics, ParsedGenerics};
 use crate::{AccountField, AccountsStruct, Field, SysvarTy, Ty};
 use proc_macro2::TokenStream;
 use quote::quote;
+use syn::Expr;
 
 // Generates the `Accounts` trait implementation.
 pub fn generate(accs: &AccountsStruct) -> proc_macro2::TokenStream {
     let name = &accs.ident;
-    let (combined_generics, trait_generics, strct_generics) = generics(accs);
-
-    // All fields without an `#[account(associated)]` attribute.
-    let non_associated_fields: Vec<&AccountField> = accs
-        .fields
-        .iter()
-        .filter(|af| !is_associated_init(af))
-        .collect();
+    let ParsedGenerics {
+        combined_generics,
+        trait_generics,
+        struct_generics,
+        where_clause,
+    } = generics(accs);
 
     // Deserialization for each field
     let deser_fields: Vec<proc_macro2::TokenStream> = accs
@@ -27,26 +26,26 @@ pub fn generate(accs: &AccountsStruct) -> proc_macro2::TokenStream {
                     quote! {
                         #[cfg(feature = "anchor-debug")]
                         ::solana_program::log::sol_log(stringify!(#name));
-                        let #name: #ty = anchor_lang::Accounts::try_accounts(program_id, accounts)?;
+                        let #name: #ty = anchor_lang::Accounts::try_accounts(program_id, accounts, ix_data)?;
                     }
                 }
                 AccountField::Field(f) => {
                     // Associated fields are *first* deserialized into
                     // AccountInfos, and then later deserialized into
                     // ProgramAccounts in the "constraint check" phase.
-                    if is_associated_init(af) {
+                    if is_pda_init(af) {
                         let name = &f.ident;
                         quote!{
                             let #name = &accounts[0];
                             *accounts = &accounts[1..];
                         }
                     } else {
-                        let name = typed_ident(&f);
+                        let name = typed_ident(f);
                         match f.constraints.is_init() {
                             false => quote! {
                                 #[cfg(feature = "anchor-debug")]
                                 ::solana_program::log::sol_log(stringify!(#name));
-                                let #name = anchor_lang::Accounts::try_accounts(program_id, accounts)?;
+                                let #name = anchor_lang::Accounts::try_accounts(program_id, accounts, ix_data)?;
                             },
                             true => quote! {
                                 #[cfg(feature = "anchor-debug")]
@@ -60,81 +59,77 @@ pub fn generate(accs: &AccountsStruct) -> proc_macro2::TokenStream {
         })
         .collect();
 
-    // Deserialization for each *associated* field. This must be after
-    // the deser_fields.
-    let deser_associated_fields: Vec<proc_macro2::TokenStream> = accs
-        .fields
-        .iter()
-        .filter_map(|af| match af {
-            AccountField::CompositeField(_s) => None,
-            AccountField::Field(f) => match is_associated_init(af) {
-                false => None,
-                true => Some(f),
-            },
-        })
-        .map(|field: &Field| constraints::generate(field))
-        .collect();
+    let constraints = generate_constraints(accs);
+    let accounts_instance = generate_accounts_instance(accs);
 
-    // Constraint checks for each account fields.
-    let access_checks: Vec<proc_macro2::TokenStream> = non_associated_fields
-        .iter()
-        .map(|af: &&AccountField| match af {
-            AccountField::Field(f) => constraints::generate(f),
-            AccountField::CompositeField(s) => constraints::generate_composite(s),
-        })
-        .collect();
-
-    // Each field in the final deserialized accounts struct.
-    let return_tys: Vec<proc_macro2::TokenStream> = accs
-        .fields
-        .iter()
-        .map(|f: &AccountField| {
-            let name = match f {
-                AccountField::CompositeField(s) => &s.ident,
-                AccountField::Field(f) => &f.ident,
-            };
+    let ix_de = match &accs.instruction_api {
+        None => quote! {},
+        Some(ix_api) => {
+            let strct_inner = &ix_api;
+            let field_names: Vec<proc_macro2::TokenStream> = ix_api
+                .iter()
+                .map(|expr: &Expr| match expr {
+                    Expr::Type(expr_type) => {
+                        let field = &expr_type.expr;
+                        quote! {
+                            #field
+                        }
+                    }
+                    _ => panic!("Invalid instruction declaration"),
+                })
+                .collect();
             quote! {
-                #name
+                let mut ix_data = ix_data;
+                #[derive(anchor_lang::AnchorSerialize, anchor_lang::AnchorDeserialize)]
+                struct __Args {
+                    #strct_inner
+                }
+                let __Args {
+                    #(#field_names),*
+                } = __Args::deserialize(&mut ix_data)
+                    .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
             }
-        })
-        .collect();
+        }
+    };
+
     quote! {
-        impl#combined_generics anchor_lang::Accounts#trait_generics for #name#strct_generics {
+        #[automatically_derived]
+        impl<#combined_generics> anchor_lang::Accounts<#trait_generics> for #name<#struct_generics> #where_clause {
             #[inline(never)]
             fn try_accounts(
                 program_id: &anchor_lang::solana_program::pubkey::Pubkey,
                 accounts: &mut &[anchor_lang::solana_program::account_info::AccountInfo<'info>],
+                ix_data: &[u8],
             ) -> std::result::Result<Self, anchor_lang::solana_program::program_error::ProgramError> {
+                // Deserialize instruction, if declared.
+                #ix_de
                 // Deserialize each account.
                 #(#deser_fields)*
-                // Deserialize each associated account.
-                //
-                // Associated accounts are treated specially, because the fields
-                // do deserialization + constraint checks in a single go,
-                // whereas all other fields, i.e. the `deser_fields`, first
-                // deserialize, and then do constraint checks.
-                #(#deser_associated_fields)*
-                // Perform constraint checks on each account.
-                #(#access_checks)*
+                // Execute accounts constraints.
+                #constraints
                 // Success. Return the validated accounts.
-                Ok(#name {
-                    #(#return_tys),*
-                })
+                Ok(#accounts_instance)
             }
         }
     }
 }
 
 // Returns true if the given AccountField has an associated init constraint.
-fn is_associated_init(af: &AccountField) -> bool {
+fn is_pda_init(af: &AccountField) -> bool {
     match af {
         AccountField::CompositeField(_s) => false,
-        AccountField::Field(f) => f
-            .constraints
-            .associated
-            .as_ref()
-            .map(|f| f.is_init)
-            .unwrap_or(false),
+        AccountField::Field(f) => {
+            f.constraints
+                .associated
+                .as_ref()
+                .map(|f| f.is_init)
+                .unwrap_or(false)
+                || f.constraints
+                    .seeds
+                    .as_ref()
+                    .map(|f| f.is_init)
+                    .unwrap_or(false)
+        }
     }
 }
 
@@ -144,31 +139,31 @@ fn typed_ident(field: &Field) -> TokenStream {
     let ty = match &field.ty {
         Ty::AccountInfo => quote! { AccountInfo },
         Ty::ProgramState(ty) => {
-            let account = &ty.account_ident;
+            let account = &ty.account_type_path;
             quote! {
                 ProgramState<#account>
             }
         }
         Ty::CpiState(ty) => {
-            let account = &ty.account_ident;
+            let account = &ty.account_type_path;
             quote! {
                 CpiState<#account>
             }
         }
         Ty::ProgramAccount(ty) => {
-            let account = &ty.account_ident;
+            let account = &ty.account_type_path;
             quote! {
                 ProgramAccount<#account>
             }
         }
         Ty::Loader(ty) => {
-            let account = &ty.account_ident;
+            let account = &ty.account_type_path;
             quote! {
                 Loader<#account>
             }
         }
         Ty::CpiAccount(ty) => {
-            let account = &ty.account_ident;
+            let account = &ty.account_type_path;
             quote! {
                 CpiAccount<#account>
             }
@@ -196,3 +191,62 @@ fn typed_ident(field: &Field) -> TokenStream {
         #name: #ty
     }
 }
+
+pub fn generate_constraints(accs: &AccountsStruct) -> proc_macro2::TokenStream {
+    // All fields without an `#[account(associated)]` attribute.
+    let non_associated_fields: Vec<&AccountField> =
+        accs.fields.iter().filter(|af| !is_pda_init(af)).collect();
+
+    // Deserialization for each *associated* field. This must be after
+    // the inital extraction from the accounts slice and before access_checks.
+    let init_associated_fields: Vec<proc_macro2::TokenStream> = accs
+        .fields
+        .iter()
+        .filter_map(|af| match af {
+            AccountField::CompositeField(_s) => None,
+            AccountField::Field(f) => match is_pda_init(af) {
+                false => None,
+                true => Some(f),
+            },
+        })
+        .map(constraints::generate)
+        .collect();
+
+    // Constraint checks for each account fields.
+    let access_checks: Vec<proc_macro2::TokenStream> = non_associated_fields
+        .iter()
+        .map(|af: &&AccountField| match af {
+            AccountField::Field(f) => constraints::generate(f),
+            AccountField::CompositeField(s) => constraints::generate_composite(s),
+        })
+        .collect();
+
+    quote! {
+        #(#init_associated_fields)*
+        #(#access_checks)*
+    }
+}
+
+pub fn generate_accounts_instance(accs: &AccountsStruct) -> proc_macro2::TokenStream {
+    let name = &accs.ident;
+    // Each field in the final deserialized accounts struct.
+    let return_tys: Vec<proc_macro2::TokenStream> = accs
+        .fields
+        .iter()
+        .map(|f: &AccountField| {
+            let name = match f {
+                AccountField::CompositeField(s) => &s.ident,
+                AccountField::Field(f) => &f.ident,
+            };
+            quote! {
+                #name
+            }
+        })
+        .collect();
+
+    quote! {
+        #name {
+            #(#return_tys),*
+        }
+    }
+}
